@@ -3,10 +3,11 @@
 from pythran.analyses.aliases import Aliases
 from pythran.analyses.global_declarations import GlobalDeclarations
 from pythran.passmanager import ModuleAnalysis
-from pythran.tables import modules
+from pythran.tables import MODULES
 import pythran.intrinsic as intrinsic
 
-import ast
+import gast as ast
+from functools import reduce
 
 
 class ArgumentReadOnce(ModuleAnalysis):
@@ -56,16 +57,16 @@ class ArgumentReadOnce(ModuleAnalysis):
         self.node_to_functioneffect = dict()
         super(ArgumentReadOnce, self).__init__(Aliases, GlobalDeclarations)
 
-    def prepare(self, node, ctx):
+    def prepare(self, node):
         """
         Initialise arguments effects as this analysis in inter-procedural.
 
         Initialisation done for Pythonic functions and default values set for
         user defined functions.
         """
-        super(ArgumentReadOnce, self).prepare(node, ctx)
+        super(ArgumentReadOnce, self).prepare(node)
         # global functions init
-        for n in self.global_declarations.itervalues():
+        for n in self.global_declarations.values():
             fe = ArgumentReadOnce.FunctionEffects(n)
             self.node_to_functioneffect[n] = fe
             self.result.add(fe)
@@ -73,7 +74,7 @@ class ArgumentReadOnce(ModuleAnalysis):
         # Pythonic functions init
         def save_effect(module):
             """ Recursively save read once effect for Pythonic functions. """
-            for intr in module.itervalues():
+            for intr in module.values():
                 if isinstance(intr, dict):  # Submodule case
                     save_effect(intr)
                 else:
@@ -83,15 +84,16 @@ class ArgumentReadOnce(ModuleAnalysis):
                     if isinstance(intr, intrinsic.Class):  # Class case
                         save_effect(intr.fields)
 
-        for m in modules:
-            save_effect(modules[m])
+        for module in MODULES.values():
+            save_effect(module)
 
-    def run(self, node, ctx):
-        ModuleAnalysis.run(self, node, ctx)
-        for fun in self.result:
-            for i in xrange(len(fun.read_effects)):
+    def run(self, node):
+        result = super(ArgumentReadOnce, self).run(node)
+        for fun in result:
+            for i in range(len(fun.read_effects)):
                 self.recursive_weight(fun, i, set())
-        return {f.func: f.read_effects for f in self.result}
+        self.result = {f.func: f.read_effects for f in result}
+        return self.result
 
     def recursive_weight(self, function, index, predecessors):
         # TODO : Find out why it happens in some cases
@@ -115,7 +117,7 @@ class ArgumentReadOnce(ModuleAnalysis):
         while isinstance(node, ast.Subscript):
             node = node.value
         if node in self.aliases:
-            for n_alias in self.aliases[node].aliases:
+            for n_alias in self.aliases[node]:
                 try:
                     return self.current_function.func.args.args.index(n_alias)
                 except ValueError:
@@ -127,11 +129,8 @@ class ArgumentReadOnce(ModuleAnalysis):
         return lambda ctx: effect if index == ctx.index else 0
 
     def generic_visit(self, node):
-        lambdas = map(self.visit, ast.iter_child_nodes(node))
-        if lambdas:
-            return lambda ctx: sum(l(ctx) for l in lambdas)
-        else:
-            return lambda ctx: 0
+        lambdas = [self.visit(child) for child in ast.iter_child_nodes(node)]
+        return lambda ctx: sum(l(ctx) for l in lambdas)
 
     def visit_FunctionDef(self, node):
         self.current_function = self.node_to_functioneffect[node]
@@ -160,22 +159,22 @@ class ArgumentReadOnce(ModuleAnalysis):
     def visit_For(self, node):
         iter_local = self.local_effect(node.iter, 1)
         iter_deps = self.visit(node.iter)
-        body_deps = map(self.visit, node.body)
-        else_deps = map(self.visit, node.orelse)
+        body_deps = [self.visit(stmt) for stmt in node.body]
+        else_deps = [self.visit(stmt) for stmt in node.orelse]
         return lambda ctx: iter_local(ctx) + iter_deps(ctx) + 2 * sum(
             l(ctx) for l in body_deps) + sum(l(ctx) for l in else_deps)
 
     def visit_While(self, node):
         test_deps = self.visit(node.test)
-        body_deps = map(self.visit, node.body)
-        else_deps = map(self.visit, node.orelse)
+        body_deps = [self.visit(stmt) for stmt in node.body]
+        else_deps = [self.visit(stmt) for stmt in node.orelse]
         return lambda ctx: test_deps(ctx) + 2 * sum(
             l(ctx) for l in body_deps) + sum(l(ctx) for l in else_deps)
 
     def visit_If(self, node):
         test_deps = self.visit(node.test)
-        body_deps = map(self.visit, node.body)
-        else_deps = map(self.visit, node.orelse)
+        body_deps = [self.visit(stmt) for stmt in node.body]
+        else_deps = [self.visit(stmt) for stmt in node.orelse]
         return lambda ctx: test_deps(ctx) + max(sum(
             l(ctx) for l in body_deps), sum(l(ctx) for l in else_deps))
 
@@ -186,15 +185,15 @@ class ArgumentReadOnce(ModuleAnalysis):
         for i, arg in enumerate(node.args):
             n = self.argument_index(arg)
             if n >= 0:
-                func_aliases = self.aliases[node].state[
-                    Aliases.access_path(node.func)]
+                func_aliases = self.aliases[node.func]
 
                 # expand argument if any
                 func_aliases = reduce(
                     lambda x, y: x + (
-                        self.node_to_functioneffect.keys()  # all functions
-                        if (isinstance(y, ast.Name)
-                            and self.argument_index(y) >= 0)
+                        # all functions
+                        list(self.node_to_functioneffect.keys())
+                        if (isinstance(y, ast.Name) and
+                            self.argument_index(y) >= 0)
                         else [y]),
                     func_aliases,
                     list())
@@ -204,6 +203,13 @@ class ArgumentReadOnce(ModuleAnalysis):
                     if isinstance(func_alias, ast.Call):
                         bound_name = func_alias.args[0].id
                         func_alias = self.global_declarations[bound_name]
+
+                    if func_alias is intrinsic.UnboundValue:
+                        continue
+
+                    if func_alias not in self.node_to_functioneffect:
+                        continue
+
                     func_alias = self.node_to_functioneffect[func_alias]
                     index_corres[n] = i
                     func = func_alias
